useDynLib(Dimodal, .registration=TRUE)
importFrom(stats, filter, mad, median, optimize, pnorm, quantile, runif)
importFrom(stats, sd, symnum, uniroot)
importFrom(statmod, pinvgauss, qinvgauss)
importFrom(grDevices, adjustcolor, col2rgb, rgb)
importFrom(grDevices, hcl.colors, hcl.pals, palette.colors, palette.pals)
importFrom(graphics, abline, axis, axTicks, box, hist, layout, legend, lines)
importFrom(graphics, mtext, par, points, rect, rug, text, title)
S3method(print, Dimodal)
S3method(summary, Dimodal)
S3method(plot, Dimodal)
S3method(print, Ditest)
S3method(summary, Ditest)
S3method(print, Diopt)
S3method(print, Dipeak)
S3method(summary, Dipeak)
S3method(print, Diflat)
S3method(summary, Diflat)
S3method(print, Didata)
S3method(summary, Didata)
S3method(print, Ditrack)
S3method(plot, Ditrack)
export(Dimodal, Diopt, Diopt.local, Ditrack)
export(mark.Dipeak, mark.Diflat)
export(Dipeak.test, Dipeak.critval, Diflat.test, Diflat.critval)
export(Dinrun.test, Dirunlen.test, Diexcurht.test, Dipermht.test)
export(midquantile, runs.as.rle, select.peaks, center.diw)
export(match.features, shiftID.place)


