
# Manually run test_Ditrack.Rtest because of run time.

source('../tests/test_Diopt.Rtest', echo=FALSE)
source('../tests/test_Dimodal.Rtest', echo=FALSE)
source('../tests/test_find_runs.Rtest', echo=FALSE)
source('../tests/test_find_peaks.Rtest', echo=FALSE)
source('../tests/test_find_flats.Rtest', echo=FALSE)
source('../tests/test_find_lvlsec.Rtest', echo=FALSE)
source('../tests/test_sigtests.Rtest', echo=FALSE)
source('../tests/test_utility.Rtest', echo=FALSE)

